import os
#from django.template import Context, loader
#from django.http import HttpResponseRedirect
from django.shortcuts import (
    get_object_or_404, render_to_response)
from django.utils.safestring import SafeUnicode
#from django.core.urlresolvers import reverse
from thuvien.collection.models import Volume
import pickle
from PIL import Image

PICKLE_FILE = '/home/bac/vnpf/vnpf.pkl'


class DiskNotMounted(Exception):
    def __init__(self, value):
        self.value = value
    def __unicode__(self):
        return "The disk is not mounted: %s" % self.value


def mark_safe(data):
    for key in data.keys():
        if key.endswith('hn') and data[key]:
            data[key] = SafeUnicode(data[key])
    return data


def load_pickle(fn):
    pf = open(fn, 'rb')
    data = pickle.load(pf)
    pf.close()
    return data


def _get_images(volume):
    mount = volume.storage.mount_point
    image_dir = os.path.join(mount, volume.nlvnpf, 'jpeg')
    image_path = os.path.join('site_media', os.path.split(mount)[1],
                              volume.nlvnpf, 'jpeg')
    if not os.path.exists(image_dir):
        raise DiskNotMounted(mount)
    images = sorted([i for i in os.listdir(image_dir) if i.endswith('.jpg')])
    return images, image_path, image_dir


def volume_index(request, object_id):
    volume = get_object_or_404(Volume, pk=object_id)
    try:
        images, image_path, image_dir = _get_images(volume)
        cover=images[-2],
    except DiskNotMounted, e:
        error_message = e
        print error_message
        images = []
        image_path = 'Unmounted'
        cover=None
    data = load_pickle(PICKLE_FILE)
    metadata = mark_safe(data[volume.nlvnpf])
    return render_to_response(
        'collection/volume-index.html',
        dict(object=volume,
             error_message=error_message,
             md=metadata,
             path=image_path,
             cover=cover,
             num_images=len(images)))


def page_thumbnails_frame(request, object_id):
    template = 'collection/pages-frame.html'
    return pages(request, object_id, template)


def pages(request, object_id, template=None):
    if template is None:
        template = 'collection/volume-pages.html'
    volume = get_object_or_404(Volume, pk=object_id)
    images, image_path, image_dir = _get_images(volume)
    return render_to_response(
        template,
        dict(object=volume,
             path=image_path,
             images=images))


def page(request, volume_id, page_id):
    template = 'collection/page-index.html'
    volume = get_object_or_404(Volume, pk=volume_id)
    images, image_path, image_dir = _get_images(volume)
    num_images = len(images) - 3
    page_num = int(page_id)
    print type(page_num)
    if page_num == num_images:
        next_id = ''
    else:
        next_id = unicode(int(page_num) + 1)
    if page_num == 1:
        prev_id = ''
    else:
        prev_id = unicode(page_num - 1)
    data = load_pickle(PICKLE_FILE)
    metadata = mark_safe(data[volume.nlvnpf])
    percent = int(float(page_num) / num_images * 100)
    image=images[page_num - 1]
    small_image = os.path.join(image_path, image)
    full_image = small_image.replace('jpeg', 'tiff').replace('jpg', 'tif')
    i = Image.open(os.path.join(image_dir, image))
    height, width = i.size
    return render_to_response(
        template,
        dict(object=volume,
             md=metadata,
             done=percent,
             page_id=page_id,
             next_id=next_id,
             prev_id=prev_id,
             last_id=num_images,
             path=image_path,
             small_image=small_image,
             full_image=full_image,
             height=height,
             width=width,
             midpoint = width/2))
